module EventObject
  refine Class do
    # Create events in specified class. For each event we create two methods:
    # `event` and `event!`. `event` returns an array of event listeners. `event!` emits
    # the listeners.
    #
    # @param [Array] the array of the symbols
    # @return [Symbol]
    def events(*ev)
      ev.each { |name| self.def("#{name}!") { |*a, &b| var(name, Array).fire(*a, &b) } }
      ev.map { |name| self.def(name) { var(name, Array) } }
    end

    # Define an instance method. `def` is public method.
    #
    # @param [Symbol]
    # @return [Symbol]
    def def(symbol, &b)
      define_method symbol, &b
    end

    # Get a class variable. If the variable is undefined then the variable will be initialized.
    #
    # @param symbol [Symbol] the name of the variable
    # @param init [...] default value
    # @return [...] the value
    def var(symbol, init=nil)
      class_variable_defined?(sym = "@#{symbol}") ? class_variable_get(sym) :
      class_variable_set(sym, (init.is_a?(Class) ? init.new : init))
    end
  end
end
